% This script provides examples to evaluate the transponder to reader
% receiver communication chain under the ISO 18000-63 standard.
% Author: Alirio Boaventura

%% Synthesizing transponder waveforms   
% Received constellation with DC offset and phase rotation 
clear all
nSymbs = 20000;      % Number of FM0 symbols
snrdB = 15;           % Desired signal-to-noise ratio
sps = 12;           % Nominal number of samples per symbol
txrx_filter = 0;    % Matched filter
dc_offset = 0;      % Complex DC offset
phase_shift = 0;    % Constellation phase rotation
beta = 0.5;
span = 10;
shape = 'Normal';
[data,s_rx]= GenRandFM0Frame(sps,nSymbs,snrdB,txrx_filter,dc_offset,phase_shift,beta,span,shape);
scatterplot(s_rx(10000:end-100))
title('')
text(-1.5,1.5,'BPSK, 15dB SNR, 0V offset, 0^o shift, Rect filter')
text(-1.4,-1.4,'(a)')
grid on
%print -dtiff 'C:\Users\asb4\Desktop\LIVRO\__Final Chapters\__Chapter 5\Figures\TagConst_1.tif' -r700

clear all
nSymbs=20000;            % Number of FM0 symbols
snrdB=20;                 % Desired signal-to-noise ratio
sps=12;                 % Nominal number of samples per symbol
txrx_filter=1;          % Matched filter
dc_offset=0.3+j*0.67;   % Complex DC offset
phase_shift=130;        % Constellation phase rotation
beta=0.5;
span = 10;
shape = 'Normal';
[data,s_rx]= GenRandFM0Frame(sps,nSymbs,snrdB,txrx_filter,dc_offset,phase_shift,beta,span,shape);
scatterplot(s_rx(10000:end))
title('')
text(-2.1,-1.5,'BPSK, 20dB SNR, 0.3+j0.7 offset, 130^o shift, RC filter')
text(-1,-1,'(b)')
grid on
%print -dtiff 'C:\Users\asb4\Desktop\LIVRO\__Final Chapters\__Chapter 5\Figures\TagConst_2.tif' -r700

sps = 100;
mOrder = 4;
nSymbs = 20000;
snrdB = 50;
txrx_filter = 1;
dc_offset = 0;
phase_shift = 45;
phase_sort = 0;
beta = 0.6;
span = 10;
shape = 'Normal';
[data,s_rx] = GenRandQPSKFrame(sps,nSymbs,mOrder,snrdB,txrx_filter,dc_offset,phase_shift,phase_sort,beta,span,shape);
scatterplot(s_rx)
title('')
text(0,0,'QPSK, 50dB SNR, 0V offset, 45^o shift, RC filter')
text(0,-0.1,'(c)')
grid on
%print -dtiff 'C:\Users\asb4\Desktop\LIVRO\__Final Chapters\__Chapter 5\Figures\TagConst_3.tif' -r700
figure
plot(real(s_rx),'color',[0,0,0]+0,'LineWidth',1.5)
hold on
plot(imag(s_rx),'-.','color',[0,0,0]+0.5,'LineWidth',1.5)
legend('Real','Imaginary')
text(3228,-0.049,'(e)')
axis([0.03e5 0.07e5 -0.13 0.13])
xlabel('Time (k)')
ylabel('Amplitude (V)')
%print -dtiff 'C:\Users\asb4\Desktop\LIVRO\__Final Chapters\__Chapter 5\Figures\TagConst_5.tif' -r500

sps=100;
mOrder=128;
nSymbs=20000;
snrdB=40;
txrx_filter=1;
dc_offset=0;
phase_shift=0;
phase_sort=1;
beta = 0.6;
span = 10;
shape = 'Normal';
[data,s_rx] = GenRandQPSKFrame(sps,nSymbs,mOrder,snrdB,txrx_filter,dc_offset,phase_shift,phase_sort,beta,span,shape);
scatterplot(s_rx(1000:end))
title('')
text(-0.1,0,'128-QPSK, 40dB SNR');
text(-0.1,-0.025,'0V offset, 0^o shift, RC filter');
text(-0.1,-0.1,'(d)')
grid on
%print -dtiff 'C:\Users\asb4\Desktop\LIVRO\__Final Chapters\__Chapter 5\Figures\TagConst_4.tif' -r900
figure
plot(real(s_rx),'color',[0,0,0]+0,'LineWidth',1.5)
hold on
plot(imag(s_rx),'-.','color',[0,0,0]+0.5,'LineWidth',1.5)
legend('Real','Imaginary')
text(150,-0.21,'(e)')
xlabel('Time (k)')
ylabel('Amplitude (V)')

% % Received constellation centered about the origin and aligned with the real-axis 
% clear all
% nSymbs=20000;            % Number of FM0 symbols
% snrdB=15;                 % Desired signal-to-noise ratio
% sps=12;                 % Nominal number of samples per symbol
% txrx_filter=1;          % Matched filter
% dc_offset=0;            % Complex DC offset
% phase_shift=0;          % Constellation phase rotation
% beta = 0.6;
% span = 10;
% shape = 'Square root';
% [data,s_rx] = GenRandFM0Frame(sps,nSymbs,snrdB,txrx_filter,dc_offset,phase_shift,beta,span,shape);
% scatterplot(s_rx(10000:end))
% title('')
% grid on
% %print -dtiff 'C:\Users\asb4\Desktop\LIVRO\__Final Chapters\__Chapter 6\Constel_2.tif' -r1200
% 
% % Received constellation filtered 
% % Design FIR low-pass filter
% fc=0.2;                                 % Filter cutoff frequency
% DEV=[0.01 0.01];                        % Maximum allowed passband and stopband errors 
% Order=2*sps;                            % Filter order 
% b=firceqrip(Order,fc,DEV);              % Filter coefficients
% s_rx_filt=filter(b,1,s_rx);             % Apply filter 
% scatterplot(s_rx_filt(10000:end),2)
% title('')
% %legend('Filtered')
% grid on
% %print -dtiff 'C:\Users\asb4\Desktop\LIVRO\__Final Chapters\__Chapter 6\Constel_3.tif' -r1200
% 
% % Clock recovery (using MATLAB comm.SymbolSynchronizer system object)
% symbolSync = comm.SymbolSynchronizer(...
%     'SamplesPerSymbol',sps/2, ...
%     'NormalizedLoopBandwidth',0.01, ...
%     'DampingFactor',1.0, ...
%     'TimingErrorDetector','Early-Late (non-data-aided)');
% 
% s_rx_sync = symbolSync(s_rx_filt.');
% scatterplot(s_rx_sync(10000:end),2)
% title('')
% %legend('Filtered')
% grid on
% %print -dtiff 'C:\Users\asb4\Desktop\LIVRO\__Final Chapters\__Chapter 6\Constel_4.tif' -r1200
% 

%% Frame synchronization
clear all
Gp = 250;               % Loop filter proportional gain
Gi = 250;               % Loop filter integral gain
nSymbs = 100;            % Number of FM0 symbols
sps = 4;               % Nominal number of samples per symbol
dc_offset = 0;          % Complex DC offset
phase_shift = 0;        % Constellation phase rotation
beta = 0.5;
span = 10;
zeroslagInsymb = 50;
shape = 'Normal';
snrdB = 100;           % Signal-to-noise ratio
txrx_filter = 0;       % Transmit pulse
[data,s_rx] = GenRandFM0Frame(sps,nSymbs,100,txrx_filter,dc_offset,phase_shift,beta,span,shape);
s_inphase = awgn([zeros(1,zeroslagInsymb*sps) real(s_rx).'],snrdB);
[true_start,s_corr] = SampleRateFM0FrameSync(s_inphase,sps); % Find start of payload data in received frame
true_start
true_peak = find(s_corr == max(s_corr))
snrdB = 10;           % Signal-to-noise ratio
[data,s_rx] = GenRandFM0Frame(sps,nSymbs,100,txrx_filter,dc_offset,phase_shift,beta,span,shape);
s_inphase = awgn([zeros(1,zeroslagInsymb*sps) real(s_rx).'],snrdB);
[estimated_start,s_corr] = SampleRateFM0FrameSync(s_inphase,sps); % Find start of payload data in received frame
estimated_start
estimated_peak = find(s_corr == max(s_corr))
figure
subplot(3,1,1)
plot(s_inphase,':','color',[0,0,0]+0.4,'LineWidth',1.3)
ylabel('In-phase magnitude (V)')
xlabel('Time (k)')
text(300,-1.75,'(a)')

axis([0 600 -2 2])
hold on
stem(true_start,2,'o','color',[0,0,0]+0,'LineWidth',1.5)
stem(estimated_start,-2,'x','color',[0,0,0]+0.5,'LineWidth',1.5)
legend('Received waveform','True payload data start','Estimated payload data start')
%plot(yfm0,'color',[0,0,0]+0,'LineWidth',1.3)
subplot(3,1,2)
plot(s_corr,':','color',[0,0,0]+0,'LineWidth',1.3)
hold on
stem(true_peak,1,'o','color',[0,0,0]+0,'LineWidth',1.5)
stem(estimated_peak,-1,'x','color',[0,0,0]+0.5,'LineWidth',1.5)
set(gca,'YColor','k');
ylabel('Norm. correlation score')
legend('Cross-correlation','True peak position','Estimated peak position')
xlabel('Time (k)')
text(300,-0.75,'(b)')

axis([0 600 -1 1])

% Determine preamble detection success rate using the original preamble frame (1010V1)  
nSymbs = 50;            % Number of FM0 symbols
M = 2000;               % Number of frames
snrdB=-15:0.5:15;
for n=1:length(snrdB)
    success1 = 0;
    success2 = 0;
    success3 = 0;
    for m=1:M
        % Sync in the sample space
        [data,s_rx] = GenRandFM0Frame(sps,nSymbs,100,txrx_filter,dc_offset,phase_shift,beta,span,shape);
        s_inphase = [zeros(1,zeroslagInsymb*sps) real(s_rx).'];
        [true_start,s_corr] = SampleRateFM0FrameSync(s_inphase,sps);        % Find true (noiseless) start of payload
        s_inphase = awgn(s_inphase,snrdB(n));
%         tic();
        [estimated_start,s_corr] = SampleRateFM0FrameSync(s_inphase,sps);  % Estimate start of payload in AWGN channel
        success1 = success1 + (estimated_start == true_start);
%         sample_t = toc()
        
        [data,s_rx] = GenRandFM0Frame_2(sps,nSymbs,100,txrx_filter,dc_offset,phase_shift,beta,span,shape);
        s_inphase = [zeros(1,zeroslagInsymb*sps) real(s_rx).'];
        [true_start,s_corr] = SampleRateFM0FrameSync_2(s_inphase,sps);        % Find true (noiseless) start of payload
        s_inphase = awgn(s_inphase,snrdB(n));
        [estimated_start,s_corr] = SampleRateFM0FrameSync_2(s_inphase,sps);  % Estimate start of payload in AWGN channel
        success2 = success2 + (estimated_start == true_start);
        
        [data,s_rx] = GenRandFM0Frame_3(sps,nSymbs,100,txrx_filter,dc_offset,phase_shift,beta,span,shape);
        s_inphase = [zeros(1,zeroslagInsymb*sps) real(s_rx).'];
        [true_start,s_corr] = SampleRateFM0FrameSync_3(s_inphase,sps);        % Find true (noiseless) start of payload
        s_inphase = awgn(s_inphase,snrdB(n));
        [estimated_start,s_corr] = SampleRateFM0FrameSync_3(s_inphase,sps);  % Estimate start of payload in AWGN channel
        success3 = success3 + (estimated_start == true_start);
    end
    success_mean1(n) = 100*success1/M;
    success_mean2(n) = 100*success2/M;
    success_mean3(n) = 100*success3/M;
end
success1 = [snrdB; success_mean1];
save('success_sync_originalPreamb.mat','success1','success2');
success1 = load('success_sync_originalPreamb.mat').success1;
subplot(3,1,3)
plot(success1(1,:), success1(2,:),'-','color',[0,0,0]+0,'LineWidth',1.3)
hold on
success2 = [snrdB; success_mean2];
save('success_sync_shorterPreamb.mat');
success2 = load('success_sync_shorterPreamb.mat').success2;
plot(success2(1,:), success2(2,:),':','color',[0,0,0]+0.7,'LineWidth',1.5)
success3 = [snrdB; success_mean3];
save('success_sync_longerPreamb.mat','success3');
success3 = load('success_sync_longerPreamb.mat').success3;
plot(success3(1,:), success3(2,:),'--','color',[0,0,0]+0.4,'LineWidth',1.3)
ylabel('Frame sync success rate (%)')
xlabel('SNR (dB)')
text(0,10,'(c)')
legend('Original preamble pattern','Shorter preamble pattern','Longer premable pattern')
%print -dtiff 'C:\Users\asb4\Desktop\LIVRO\__Final Chapters\__Chapter 5\Figures\Frame Sync.tif' -r1200



% Load and decode real transponder RN16 response
clear all
Gp = 250;                 % Loop filter proportional gain
Gi = 250;                 % Loop filter integral gain
nSymbs=16;                % Number of symbols to be decoded
sps=12;                   % Nominal number of Number of samples per symbol
txrx_filter = 0;          % Receive rectangular pulse shape
dc_offset = 0;            % Complex DC offset
phase_shift = 0;          % Constellation phase rotation
beta = 0.5;
span = 10;
shape = 'Normal';
s_rx=load('RN16_tag_reply_2.mat')       % Load transponder RN16 baseband response
s_rx=s_rx.x;
fc=0.4;                                 % FIR filter cutoff frequency
DEV=[0.01 0.01];                        % Maximum allowed passband and stopband errors 
Order=sps;                              % FIR filter order 
b=firceqrip(Order,fc,DEV);              % FIR filter coefficients
s_rx=filter(b,1,s_rx);                  % FIR filtering 
s_rx=s_rx(50:end)-(min(s_rx(50:end))+max(s_rx(50:end)))/2;      % Center signal about the origin
[pay_start,preamb_cross_corr] = SampleRateFM0FrameSync(s_rx,sps); % Locate payload data in the received frame
figure
plot(1000*s_rx,'-','color',[0,0,0]+0.6,'LineWidth',1.3)
xlim([20 530])
xlabel('Time (k)')
ylabel ('In-phase magnitude (mV)')
hold on
yyaxis right
plot(1200*preamb_cross_corr,'color',[0,0,0]+0,'LineWidth',1.3)
ylabel ('Correlation score')
legend('Transponder RN16 response','Correlation score')
%print -dtiff 'C:\Users\asb4\Desktop\LIVRO\__Final Chapters\__Chapter 5\Frame Sync.tif' -r1200

%  
% %% Rectangular matched filter waveforms
% clear all
% sps=12;                     % Number of samples per symbol
% nSymbs=10;                   % Number of symbols per frame
% Gp=250;
% Gi=250;
% snrdB=70;
% txrx_filter=0;
% beta=0.5;
% dc_offset=0;
% phase_shift=0;
% span = 8;
% shape = 'Normal';
% % Preamble detection
% [data,s_rx]= GenRandFM0Frame(sps,nSymbs,snrdB,txrx_filter,dc_offset,phase_shift,beta,span,shape);
% s_rx = real(s_rx);
% pay_start = SampleRateFM0FrameSync(s_rx,sps); % Find beginning of the data payload in the received frame
% 
% % Correlator (matched filter) output for '1' FM0 pulse template
% s_rx = s_rx;
% nSymbs = length(s_rx);
% fm0_pulse1 = ones(1,sps);
% n = 1;
% r0 = conv(s_rx,flip(fm0_pulse1));
% 
% % for lag = -sps:nSymb
% %     r0(n) = Xcorr(s_rx,fm0_pulse1,lag);
% %     n = n+1;
% % end
% r0 = r0(12*sps+1:end);
% r0 = r0/max(r0);
% s_rx = s_rx(12*sps+1:end);
% figure 
% plot(s_rx,'-','color',[0,0,0]+0.7,'LineWidth',2)
% hold on
% plot(r0,':','color',[0,0,0]+0,'LineWidth',2.5)
% % Correlator (matched filter) output for '0' FM0 pulse template
% s_rx = s_rx;
% nSymbs = length(s_rx);
% fm0_pulse0 = [-ones(1,sps/2) ones(1,sps/2)];
% sps = sps;
% n = 1;
% r0 = conv(s_rx,flip(fm0_pulse0));
% % for lag = -sps:nSymb
% %     r0(n) = Xcorr(s_rx,fm0_pulse0,lag);
% %     n = n+1;
% % end
% r0 = r0(12*sps+1:end);
% r0 = r0/max(r0);
% s_rx = s_rx(12*sps+1:end);
% plot(r0,'-.','color',[0,0,0]+0.4,'LineWidth',1.5)
% legend('FM0 sequence "101"','Correlation with "1"','Correlation with "0"')
% xlabel('Time (k)')
% ylabel('Normalized amplitude')
% xlim([1 3*sps])
% ylim([-1.2 1.2])
% %print -dtiff 'C:\Users\asb4\Desktop\LIVRO\__Final Chapters\Figuras - Chapter 6\Figure 6.FM0matching.tif' -r1200
% 

%% Mueller & Muller DPLL waveforms
% Time domain with rectangular pulse shape
clear all
rng(12) % For repeatable results
Gp = 250;
Gi = 250;
nSymbs = 10;         % Number of symbols per frame
sps = 100;          % Number of samples per symbol
snrdB = -4;
txrx_filter = 0;    % Use rectangular pulse shape
dc_offset = 0;      % Complex DC offset
phase_shift = 0;    % Constellation phase rotation
shape = 'Normal';
beta = 0.5;
span = 100;
delay = sps/4;      % delay of rectangular filter
[data,s_rx,s_fm0]= GenRandFM0Frame(sps,nSymbs,snrdB,txrx_filter,dc_offset,phase_shift,beta,span,shape);
s_inphase=real(s_rx);         % Demodulate in-phase component only
%s_rx_inphase(1:delay)=[];       % compensate for rx filter
[s_clk, s_match] = DPLLMnMClockRecovery(s_inphase,sps,Gp,Gi,txrx_filter,span,shape,beta);
s_match(1:delay)=[];      % Compensate for rx filter delay only (the rect tx filter has no delay)
pay_start = SymbolRateFM0FrameSync(s_clk);
dec_data = FM0SymbolDemap(s_clk,pay_start,nSymbs);
isequal(data,dec_data)
% figure
subplot(3,1,1)
plot(s_inphase./max(s_inphase),'color',[0,0,0]+0.7,'LineWidth',1.2)
% axis([1 length(s_rx_2) -5 5])
ylabel('In-phase norm. amplitude')
hold on
% yyaxis right
plot(s_match(1:end)./max(s_match),':','color',[0,0,0]+0.4,'LineWidth',2.2)
set(gca,'YColor','k');
n=sps/4:sps/2:length(s_clk)*sps/2;
plot(n,s_fm0,'ko','MarkerSize',7,'LineWidth',1.2)
stem(n,s_clk,'xk','LineWidth',1.2)
axis([1825 2775 -1.5 1.5])
xlabel('Time (k)')
% ylabel('Matched filter output, Clock (V)')
text(2310,1.25,'(a)')
legend('Received FM0 waveform','Matched filter output','Transmitted clock','Recovered clock')

% Time domain with square root raised cosine pulse shape
clear all
rng(12) % For repeatable results
Gp = 250;
Gi = 250;
nSymbs = 10;         % Number of symbols per frame
sps = 100;          % Number of samples per symbol
snrdB = -4;
txrx_filter = 1;    % Raised cosine filter
dc_offset = 0;      % Complex DC offset
phase_shift = 0;    % Constellation phase rotation
shape = 'Square root';
beta = 0.7;
span = 8;
delay = (sps/2)*span/2;
[data,s_rx,s_fm0]= GenRandFM0Frame(sps,nSymbs,snrdB,txrx_filter,dc_offset,phase_shift,beta,span,shape);
s_inphase=real(s_rx);     % Demodulate in-phase component only
s_inphase(1:delay)=[];    % Compensate for tx filter delay
[s_clk, s_match] = DPLLMnMClockRecovery(s_inphase,sps,Gp,Gi,txrx_filter,span,shape,beta);
pay_start = SymbolRateFM0FrameSync(s_clk);
dec_data = FM0SymbolDemap(s_clk,pay_start,nSymbs);
isequal(data,dec_data)
% figure
subplot(3,1,2)
plot(s_inphase./max(s_inphase),'color',[0,0,0]+0.7,'LineWidth',1.2)
% axis([1 length(s_rx_2) -5 5])
ylabel('In-phase norm. amplitude')
hold on
s_match(1:delay)=[];    % Compensate for rx filter delay
plot(s_match./max(s_match),':','color',[0,0,0]+0.4,'LineWidth',2.2)
set(gca,'YColor','k');
n=0:sps/2:length(s_clk)*sps/2-delay;
plot(n,s_fm0(1:length(n)),'ko','MarkerSize',7,'LineWidth',1.2)
stem(n,s_clk(delay/(sps/2):end),'kx','LineWidth',1.2)
axis([1800 2750 -1.5 1.5])
xlabel('Time (k)')
text(2275,1.25,'(b)')
% legend('Received FM0 waveform','Matched filter output','Transmitted clock','Recovered clock')

% Time domain with tx square root raised cosine and no receiver filter
% For this case, bypass the receiver filter by
% doing r = s_rx in function DPLLMnMClockRecovery()
clear all
%figure
rng(12) % For repeatable results
Gp = 250;
Gi = 250;
nSymbs = 10;      % Number of symbols per frame
sps = 100;        % Number of samples per symbol
snrdB = -4;
tx_filter = 0;    % Use rectangular pulse shape
rx_filter = 2;    % Use no filter at the receiver
dc_offset = 0;    % Complex DC offset
phase_shift = 0;  % Constellation phase rotation
shape = 'Square root';
beta = 0.7;
span = 8;
[data,s_rx,s_fm0]= GenRandFM0Frame(sps,nSymbs,snrdB,tx_filter,dc_offset,phase_shift,beta,span,shape);
s_inphase=real(s_rx);           % Demodulate in-phase component only
% s_rx_inphase(1:delay)=[];     % Compensate for tx filter delay
[s_clk, s_match] = DPLLMnMClockRecovery(s_inphase,sps,Gp,Gi,rx_filter,span,shape,beta);
% filter delay is null
%s_match(1:delay)=[];
pay_start = SymbolRateFM0FrameSync(s_clk);
dec_data = FM0SymbolDemap(s_clk,pay_start,nSymbs);
isequal(data,dec_data)
s_inphase(1:pay_start*sps/2)=[];    % Get data starting at payL_start
s_match(1:pay_start*sps/2)=[];      % Get data starting at payL_start
%figure
subplot(3,1,3)
plot(s_inphase./max(s_inphase),'color',[0,0,0]+0.7,'LineWidth',1.2)
% axis([1 length(s_rx_2) -5 5])
ylabel('In-phase norm. amplitude')
hold on
plot(s_match./max(s_match),':','color',[0,0,0]+0.4,'LineWidth',2.2)
% set(gca,'YColor','k');
s_fm0(1:36)=[];
n=0:sps/2:(length(s_fm0)-1)*sps/2;
plot(n,s_fm0,'ko','MarkerSize',7,'LineWidth',1.2)
s_clk(1:pay_start-1)=[];
n=0:sps/2:(length(s_clk)-1)*sps/2;
stem(n,s_clk,'xk','LineWidth',1.2)
axis([0 nSymbs*sps-sps/2 -1.5 1.5])
xlabel('Time (k)')
text(465,1.25,'(c)')
%print -dtiff 'C:\Users\ajsb\Documents\RFID-Book\Final-Manuscript\Final-Figures\Chapter-5\Figure 5.16.tif' -r1200

%% Constellations
clear all
Gp=250;
Gi=250;
nSymbs=10000;           % Number of symbols per frame
sps=100;                % Number of samples per symbol
snrdB=6;
delay=0;
txrx_filter=1;          % Matched filter
dc_offset=0;            % Complex DC offset
phase_shift=0;          % Constellation phase rotation
span=10;
shape='Square root';
beta=0.5;
[data,s_rx]= GenRandFM0Frame(sps,nSymbs,snrdB,txrx_filter,dc_offset,phase_shift,beta,span,shape);
s_inphase=real(s_rx);    % Demodulate in-phase component only
[s_clk, s_match] = DPLLMnMClockRecovery(s_inphase,sps,Gp,Gi,txrx_filter,span,shape,beta);
% figure; plot(s_rx_inphase);hold on; plot(imag(s_rx))
pay_start = SymbolRateFM0FrameSync(s_clk);
dec_data = FM0SymbolDemap(s_clk,pay_start,nSymbs);
isequal(data,dec_data)
figure
plot(s_rx,'.','color',[0,0,0]+0.7)
axis([-0.5 0.5 -0.5 0.5])
grid on
xlabel('In-Phase')
ylabel('Quadrature')
hold on
% Low-pass FIR filter received signal
fc=0.1;                                 % FIR filter cutoff frequency
DEV=[0.01 0.01];                        % Maximum allowed passband and stopband errors 
Order=sps;                              % FIR filter order 
b=firceqrip(Order,fc,DEV);              % FIR filter coefficients
s_rx_filt=filter(b,1,s_rx);                  % FIR filtering 
plot(s_rx_filt,'.','color',[0,0,0]+0.4)
% Clock recovery (using MATLAB comm.SymbolSynchronizer system object)
symbolSync = comm.SymbolSynchronizer(...
    'SamplesPerSymbol',sps/2, ...
    'NormalizedLoopBandwidth',0.01, ...
    'DampingFactor',1.0, ...
    'TimingErrorDetector','Mueller-Muller (decision-directed)');
s_rx_sync = symbolSync(s_rx_filt.');
plot(s_rx_sync(200:end-200),'.','color',[0,0,0]+0)
legend('Received signal','After filtering','After timing recovery')
x0 = 10;
y0 = 10;
width = 500;
height = 500;
set(gcf,'position',[x0,y0,width,height]);
%print -dtiff 'C:\Users\asb4\Desktop\LIVRO\__Final Chapters\__Chapter 5\Figures\Clock recovery3.tif' -r1200

%% Eye diagrams
% Rectangular pulse macthed filter
clear all
Gp = 250;
Gi = 250;
nSymbs = 100;            % Number of symbols per frame
sps = 100;              % Number of samples per symbol
snrdB = 6;
txrx_filter = 0;        % Apply transmit filter to the FM0 waveform
dc_offset = 0;          % Complex DC offset
phase_shift = 0;        % Constellation phase rotation
span = 8;
shape = 'normal';
beta = 0.5;
[data,s_rx]= GenRandFM0Frame(sps,nSymbs,snrdB,txrx_filter,dc_offset,phase_shift,beta,span,shape);
s_inphase=real(s_rx);    % Demodulate in-phase component only
[s_clk, s_match] = DPLLMnMClockRecovery(s_inphase,sps,Gp,Gi,txrx_filter,span,shape,beta);
pay_start = SymbolRateFM0FrameSync(s_clk);
dec_data = FM0SymbolDemap(s_clk,pay_start,nSymbs);
isequal(data,dec_data)
s_rx_2=real(s_rx); 
eyediagram(s_rx_2(sps/2:9950),sps,sps,sps/4);
ylabel('In-phase magnitude (V)')
xlabel('Time (k)')
title('')
grid on
ylim([-2.2 2.2])
%print -dtiff 'C:\Users\asb4\Desktop\LIVRO\__Final Chapters\__Chapter 5\eye1.tif' -r1200
eyediagram(s_match(sps/2:9950),sps/2,sps,sps/4);
ylabel('In-phase magnitude (V)')
title('')
grid on
ylim([-1.2 1.2])
xlabel('Time (k)')
%print -dtiff 'C:\Users\asb4\Desktop\LIVRO\__Final Chapters\__Chapter 5\eye2.tif' -r1200

% Square root raised cosine matched filtering
clear all
Gp = 250;
Gi = 250;
nSymbs = 100;                % Number of symbols per frame
sps = 100;                  % Number of samples per symbol
snrdB = 6;
txrx_filter = 1;            % Matched filter
dc_offset = 0;              % Complex DC offset
phase_shift = 0;            % Constellation phase rotation
span = 16;
shape = 'Square root';
beta = 0.7;
delay = (sps/2)*span/2;
[data,s_rx]= GenRandFM0Frame(sps,nSymbs,snrdB,txrx_filter,dc_offset,phase_shift,beta,span,shape);
s_inphase=real(s_rx);         % Demodulate in-phase component only
s_inphase(1:delay)=[];        % Compensate for tx filter delay
[s_clk, s_match] = DPLLMnMClockRecovery(s_inphase,sps,Gp,Gi,txrx_filter,span,shape,beta);
s_match(1:delay)=[];      % Compensate for rx filter delay
pay_start = SymbolRateFM0FrameSync(s_clk);
dec_data = FM0SymbolDemap(s_clk,pay_start,nSymbs);
isequal(data,dec_data)
eyediagram(s_inphase(1:11500),sps/2,sps,sps/4);
ylabel('In-phase magnitude (V)')
xlabel('Time (k)')
title('')
grid on
ylim([-0.35 0.35])
%print -dtiff 'C:\Users\asb4\Desktop\LIVRO\__Final Chapters\__Chapter 5\eye3.tif' -r1200
eyediagram(s_match(100:11500),sps/2,sps,sps/4);
ylabel('In-phase magnitude (V)')
title('')
grid on
ylim([-1.2 1.2])
xlabel('Time (k)')
%print -dtiff 'C:\Users\asb4\Desktop\LIVRO\__Final Chapters\__Chapter 5\eye4-2.tif' -r1200

%% Reception success rate
% Baseline comparison between theoretical error probability of BPSK demodulation
% and bit error rate (BER) of FM0 demodlation using hard decision in ideal AWGN channel 
clear all
Gp = 250;         % Loop filter proportional gain
Gi = 250;         % Loop filter integral gain
sps = 12;         % Samples per FM0 symbol
nSymbs = 200;      % Symbols per frame
M = 10000;        % Number of frames
EbN0 = 0:0.5:25;
dc_offset = 0;
phase_shift = 0;
delay = 0;
beta = 0.5;
span = 8;
shape = 'Normal';
tx_filter=0;
k = 1; % Bits per symbol
snrdB = EbN0+10*log10(k)-10*log10(sps/2);
% Success rate with ideal sampling and hard decision in ideal AWGN channel 
for n = 1:length(snrdB)
    success_sum = 0;
    for m = 1:M
        [data,s_rx,s_fm0] = GenRandFM0Frame(sps,nSymbs,snrdB(n),tx_filter,dc_offset,phase_shift,beta,span,shape);
        s_decimate = s_rx(1:sps/2:end);    % Decimate received signal at a factor of sps/2 to get BPSK signal  
        % Decode received signal using hard decision 
        dec_bpsk = zeros(1,length(s_fm0));
        for k = 1:length(s_decimate)
            dec_bpsk(k) = (real(s_decimate(k)) > 0);    % Decode received signal as a BPSK signal 
            s_bpsk(k) = s_fm0(k) > 0;                   % Demap original FM0 signal into BPSK symbols for compairison 
        end
        success_sum = success_sum+isequal(s_bpsk,dec_bpsk);
    end
   success_mean(n) = 100*success_sum/M;
end
success = [EbN0; success_mean];
%save('success_HardDecisionFinal.mat','success');
success = load('success_HardDecisionFinal.mat').success;
figure(1)
plot(success(1,:), success(2,:),'color',[0,0,0]+0.7,'LineWidth',1.2)
hold on
S=1

clear all
Gp = 250;         % Loop filter proportional gain
Gi = 250;         % Loop filter integral gain
sps = 12;         % Samples per FM0 symbol
nSymbs = 200;      % Symbols per frame
M = 10000;        % Number of frames
EbN0 = 0:0.5:25;
dc_offset = 0;
phase_shift = 0;
delay = 0;
beta = 0.5;
span = 8;
shape = 'Normal';
txrx_filter = 0;
k = 0.5; % Bits per symbol (1 MF0 symbol = 2 BPSK symbols)
snrdB = EbN0+10*log10(k)-10*log10(sps/2);
for n = 1:length(snrdB)
    success_sum = 0;
    for m = 1:M
        [data,s_rx]= GenRandFM0Frame(sps,nSymbs,snrdB(n),txrx_filter,dc_offset,phase_shift,beta,span,shape);
        s_inphase=real(s_rx);
        [s_clk,s_match] = DPLLMnMClockRecovery(s_inphase,sps,Gp,Gi,txrx_filter,span,shape,beta);
        pay_start = SymbolRateFM0FrameSync(s_clk);
        dec_data = FM0SymbolDemap(s_clk,pay_start,nSymbs);
        success_sum = success_sum+isequal(data,dec_data);
    end
    success_mean(n) = 100*success_sum/M;
end
success = [EbN0; success_mean];
%save('success_DPLL_RectfilerFinal.mat','success');
success = load('success_DPLL_RectfilerFinal.mat').success;
figure(1)
plot(success(1,:), success(2,:))
S=2

clear all
Gp = 250;         % Loop filter proportional gain
Gi = 250;         % Loop filter integral gain
sps = 12;         % Samples per FM0 symbol
nSymbs = 200;      % Symbols per frame
M = 10000;        % Number of frames
EbN0 = 0:0.5:25;
dc_offset = 0;
phase_shift = 0;
delay = 0;
beta = 0.5;
span = 8;
shape = 'Normal';
txrx_filter=1;
k = 0.5; % Bits per symbol (1 MF0 symbol = 2 BPSK symbols)
snrdB = EbN0+10*log10(k)-10*log10(sps/2);
for n = 1:length(snrdB)
    success_sum = 0;
    for m = 1:M
        [data,s_rx]= GenRandFM0Frame(sps,nSymbs,snrdB(n),txrx_filter,dc_offset,phase_shift,beta,span,shape);
        s_inphase=real(s_rx);
        [s_clk, s_match] = DPLLMnMClockRecovery(s_inphase,sps,Gp,Gi,txrx_filter,span,shape,beta);
        pay_start = SymbolRateFM0FrameSync(s_clk);
        dec_data = FM0SymbolDemap(s_clk,pay_start,nSymbs);
        success_sum = success_sum+isequal(data,dec_data);
    end
    success_mean(n) = 100*success_sum/M;
end
success = [EbN0; success_mean];
%save('success_DPLL_RCfilterFinal.mat','success');
success = load('success_DPLL_RCfilterFinal.mat').success;
figure(1)
plot(success(1,:), success(2,:))
S=3
 
clear all
Gp = 250;         % Loop filter proportional gain
Gi = 250;         % Loop filter integral gain
sps = 12;         % Samples per FM0 symbol
nSymbs = 200;      % Symbols per frame
M = 10000;        % Number of frames
EbN0 = 0:0.5:25;
dc_offset = 0;
phase_shift = 0;
delay = 0;
beta = 0.5;
span = 8;
shape = 'Square root';
txrx_filter=1;
k = 0.5; % Bits per symbol (1 MF0 symbol = 2 BPSK symbols)
snrdB = EbN0+10*log10(k)-10*log10(sps/2);
for n = 1:length(snrdB)
    success_sum = 0;
    for m = 1:M
        [data,s_rx]= GenRandFM0Frame(sps,nSymbs,snrdB(n),txrx_filter,dc_offset,phase_shift,beta,span,shape);
        s_inphase=real(s_rx);
        [s_clk, s_match] = DPLLMnMClockRecovery(s_inphase,sps,Gp,Gi,txrx_filter,span,shape,beta);
        pay_start = SymbolRateFM0FrameSync(s_clk);
        dec_data = FM0SymbolDemap(s_clk,pay_start,nSymbs);
        success_sum = success_sum+isequal(data,dec_data);
    end
   success_mean(n) = 100*success_sum/M;
end
success = [EbN0; success_mean];
%save('success_DPLL_SRRCfilterFinal.mat','success');
success = load('success_DPLL_SRRCfilterFinal.mat').success;
figure(1)
plot(success(1,:), success(2,:))
S=4
 
% DPLL with no filter in the receiver. For this case, bypass the receive filter
% by doing r = s_rx in the function DPLLMnMClockRecovery()
EbN0 = 0:0.5:25;
tx_filter = 0;
rx_filter = 3;
k = 0.5; % Bits per symbol (1 MF0 symbol = 2 BPSK symbols)
snrdB = EbN0+10*log10(k)-10*log10(sps/2);
for n=1:length(snrdB)
    success_sum = 0;
    for m=1:M
        [data,s_rx]= GenRandFM0Frame(sps,nSymbs,snrdB(n),tx_filter,dc_offset,phase_shift,beta,span,shape);
        s_inphase=real(s_rx);
        [s_clk, s_match] = DPLLMnMClockRecovery(s_inphase,sps,Gp,Gi,rx_filter,span,shape,beta);
        payL_start = SymbolRateFM0FrameSync(s_clk);
        dec_data = FM0SymbolDemap(s_clk,payL_start,nSymbs);
        success_sum = success_sum+isequal(data,dec_data);
    end
   success_mean(n)=100*success_sum/M;
end
success = [EbN0; success_mean];
%save('success_DPLL_NoRxfilterFinal.mat','success');
success = load('success_DPLL_NoRxfilterFinal.mat').success;
figure(1)
plot(success(1,:), success(2,:))
S=5
 
% Zero crossing demodulation
clear all
Gp=250;         % Loop filter proportional gain
Gi=250;         % Loop filter integral gain
sps=12;         % Samples per FM0 symbol
nSymbs=200;      % Symbols per frame
M=10000;        % Number of frames
dc_offset=0;
phase_shift=0;
delay=0;
beta=0.5;
span=8;
shape='Normal';
EbN0 = 0:0.5:25;
txrx_filter=0;
k = 0.5; % Bits per symbol (1 MF0 symbol = 2 BPSK symbols)
snrdB = EbN0+10*log10(k)-10*log10(sps/2);
for n=1:length(snrdB)
    success_sum = 0;
    for m=1:M
        [data,s_rx]= GenRandFM0Frame(sps,nSymbs,snrdB(n),txrx_filter,dc_offset,phase_shift,beta,span,shape);
        s_inphase=real(s_rx);
        dec_data=ZCrossFM0Demod(s_inphase,sps,nSymbs);
        success_sum = success_sum+isequal(data,dec_data);
        success_mean(n)=100*success_sum/M;
    end
   success_mean(n)=100*success_sum/M;
end
success = [EbN0; success_mean];
%save('success_ZC1_rectfilterFinal.mat','success');
success = load('success_ZC1_rectfilterFinal.mat').success;
figure(1)
plot(success(1,:), success(2,:))
S=6
 
%% Bit Error Rate
% Ideal sampling and hard decision 
clear all
Gp = 250;         % Loop filter proportional gain
Gi = 250;         % Loop filter integral gain
sps = 12;         % Samples per FM0 symbol
nSymbs = 200;     % Symbols per frame
EbN0 = -5:0.5:20;
dc_offset = 0;
phase_shift = 0;
delay = 0;
beta = 0.5;
span = 8;
shape = 'Normal';
tx_filter=0;
k = 1; % Bits per symbol (BPSK)
snrdB = EbN0+10*log10(k)-10*log10(sps/2);

% Theoretical BPSK error probability in ideal AWGN channel  
% with perfect sampling and hard decision decoding
snrLin = 10.^(snrdB./10); % Convert SNR to linear
Pbpsk = qfunc(sqrt(2*snrLin)); 
figure(2)
semilogy(EbN0,Pbpsk)
hold on

% BPSK demodulation using hard decision at ideal sampling instants 
for n = 1:length(snrdB)
    % Reset the error and bit counters
    numErrs = 0;
    numBits = 0;
    while(numErrs < 200 && numBits < 1e7)
        % Generate arbitrary BPSK data
        [data,s_rx,s_fm0] = GenRandFM0Frame(sps,nSymbs,snrdB(n),tx_filter,dc_offset,phase_shift,beta,span,shape);
        s_decimate = s_rx(1:sps/2:end);    % Decimate received signal at a factor of sps/2 to get BPSK signal  
        % Decode received signal using hard decision 
        dec_bpsk = zeros(1,length(s_fm0));
        for m = 1:length(s_decimate)
            dec_bpsk(m) = (real(s_decimate(m)) > 0);    % Decode received signal as a BPSK signal 
            s_bpsk(m) = s_fm0(m) > 0;                   % Demap original FM0 signal into BPSK symbols for compairison 
        end
        % Calculate the number of bit errors
        nErrors = biterr(s_bpsk,dec_bpsk);
        % Increment the error and bit counters
        numErrs = numErrs + nErrors;
        numBits = numBits + length(dec_bpsk);
    end
    % Estimate the BER
    ber_mean(n) = numErrs/numBits;
end
figure(2)
ber = [EbN0; ber_mean];
%save('ber_HardDecisionFinal.mat','ber');
ber = load('ber_HardDecisionFinal.mat').ber;
semilogy(ber(1,:), ber(2,:),'o')
%axis([-10 10 1e-5 1])
B=1
 
% DPLL with tx/rx rectangular matched filter
clear all
Gp = 250;         % Loop filter proportional gain
Gi = 250;         % Loop filter integral gain
sps = 12;         % Samples per FM0 symbol
nSymbs = 200;      % Symbols per frame
EbN0 = -5:0.5:25;
dc_offset = 0;
phase_shift = 0;
delay = 0;
beta = 0.5;
span = 8;
shape = 'Normal';
txrx_filter = 0;
k = 0.5; % Bits per symbol (1 bit (FM0 symbol) = 2 BPSK symbols)
snrdB = EbN0+10*log10(k)-10*log10(sps/2);

% Find true start of frame just once oustside the loop as BER is computed only on payload data
[data,s_rx,s_fm0] = GenRandFM0Frame(sps,nSymbs,100,txrx_filter,dc_offset,phase_shift,beta,span,shape);
s_inphase=real(s_rx);
% Decode received signal using hard decision 
[s_clk,s_match] = DPLLMnMClockRecovery(s_inphase,sps,Gp,Gi,txrx_filter,span,shape,beta);
pay_start = SymbolRateFM0FrameSync(s_clk);  % 
for n = 1:length(snrdB)
    % Reset the error and bit counters
    numErrs = 0;
    numBits = 0;
    while(numErrs < 200 && numBits < 1e7)
        % Generate arbitrary FM0 data
        [data,s_rx,s_fm0] = GenRandFM0Frame(sps,nSymbs,snrdB(n),txrx_filter,dc_offset,phase_shift,beta,span,shape);
        s_inphase=real(s_rx);
        % Decode received signal
        [s_clk,s_match] = DPLLMnMClockRecovery(s_inphase,sps,Gp,Gi,txrx_filter,span,shape,beta);
        dec_data = FM0SymbolDemap(s_clk,pay_start,nSymbs);
        % Calculate the number of bit errors
        nErrors = biterr(data,dec_data);
        % Increment the error and bit counters
        numErrs = numErrs + nErrors;
        numBits = numBits + nSymbs; % 1 FM0 symbol = 1 bit
    end
    % Estimate the BER
    ber_mean(n) = numErrs/numBits;
end
figure(2)
ber = [EbN0; ber_mean];
%save('ber_DPLL_RectfilerFinal.mat','ber');
ber = load('ber_DPLL_RectfilerFinal.mat').ber;
semilogy(ber(1,:), ber(2,:),'o')
B=2
 
% DPLL with raised cosine filter in the transmitter and receiver
clear all
Gp = 250;         % Loop filter proportional gain
Gi = 250;         % Loop filter integral gain
sps = 12;         % Samples per FM0 symbol
nSymbs = 200;      % Symbols per frame
EbN0 = -5:0.5:25;
dc_offset = 0;
phase_shift = 0;
delay = 0;
beta = 0.5;
span = 8;
shape = 'Normal';
txrx_filter = 1;
k = 0.5; % Bits per symbol (1 bit (FM0 symbol) = 2 BPSK symbols)
snrdB = EbN0+10*log10(k)-10*log10(sps/2);

% Find true start of frame just once oustside the loop as BER is computed only on payload data
[data,s_rx,s_fm0] = GenRandFM0Frame(sps,nSymbs,100,txrx_filter,dc_offset,phase_shift,beta,span,shape);
s_inphase=real(s_rx);
% Decode received signal using hard decision 
[s_clk,s_match] = DPLLMnMClockRecovery(s_inphase,sps,Gp,Gi,txrx_filter,span,shape,beta);
pay_start = SymbolRateFM0FrameSync(s_clk);  % 
for n = 1:length(snrdB)
    % Reset the error and bit counters
    numErrs = 0;
    numBits = 0;
    while(numErrs < 200 && numBits < 1e7)
        % Generate arbitrary FM0 data
        [data,s_rx,s_fm0] = GenRandFM0Frame(sps,nSymbs,snrdB(n),txrx_filter,dc_offset,phase_shift,beta,span,shape);
        s_inphase=real(s_rx);
        % Decode received signal
        [s_clk,s_match] = DPLLMnMClockRecovery(s_inphase,sps,Gp,Gi,txrx_filter,span,shape,beta);
        dec_data = FM0SymbolDemap(s_clk,pay_start,nSymbs);
        % Calculate the number of bit errors
        nErrors = biterr(data,dec_data);
        % Increment the error and bit counters
        numErrs = numErrs + nErrors;
        numBits = numBits + nSymbs; % 1 FM0 symbol = 1 bit
    end
    % Estimate the BER
    ber_mean(n) = numErrs/numBits;
end
figure(2)
ber = [EbN0; ber_mean];
%save('ber_DPLL_RCfilterFinal.mat','ber');
ber = load('ber_DPLL_RCfilterFinal.mat').ber;
semilogy(ber(1,:), ber(2,:),'o')
B=2 

% DPLL with matched square root raised cosine filter
clear all
Gp = 250;         % Loop filter proportional gain
Gi = 250;         % Loop filter integral gain
sps = 12;         % Samples per FM0 symbol
nSymbs = 200;      % Symbols per frame
EbN0 = -5:0.5:25;
dc_offset = 0;
phase_shift = 0;
delay = 0;
beta = 0.5;
span = 8;
shape = 'Square root';
txrx_filter = 1;
k = 0.5; % Bits per symbol (1 bit (FM0 symbol) = 2 BPSK symbols)
snrdB = EbN0+10*log10(k)-10*log10(sps/2);

% Find true start of frame just once oustside the loop as BER is computed only on payload data
[data,s_rx,s_fm0] = GenRandFM0Frame(sps,nSymbs,100,txrx_filter,dc_offset,phase_shift,beta,span,shape);
s_inphase=real(s_rx);
% Decode received signal using hard decision 
[s_clk,s_match] = DPLLMnMClockRecovery(s_inphase,sps,Gp,Gi,txrx_filter,span,shape,beta);
pay_start = SymbolRateFM0FrameSync(s_clk);  % 
for n = 1:length(snrdB)
    % Reset the error and bit counters
    numErrs = 0;
    numBits = 0;
    while(numErrs < 200 && numBits < 1e7)
        % Generate arbitrary FM0 data
        [data,s_rx,s_fm0] = GenRandFM0Frame(sps,nSymbs,snrdB(n),txrx_filter,dc_offset,phase_shift,beta,span,shape);
        s_inphase=real(s_rx);
        % Decode received signal
        [s_clk,s_match] = DPLLMnMClockRecovery(s_inphase,sps,Gp,Gi,txrx_filter,span,shape,beta);
        dec_data = FM0SymbolDemap(s_clk,pay_start,nSymbs);
        % Calculate the number of bit errors
        nErrors = biterr(data,dec_data);
        % Increment the error and bit counters
        numErrs = numErrs + nErrors;
        numBits = numBits + nSymbs; % 1 FM0 symbol = 1 bit
    end
    % Estimate the BER
    ber_mean(n) = numErrs/numBits;
end
figure(2)
ber = [EbN0; ber_mean];
%save('ber_DPLL_SRRCfilterFinal.mat','ber');
ber = load('ber_DPLL_SRRCfilterFinal.mat').ber;
semilogy(ber(1,:), ber(2,:),'o')
B=3 

% DPLL with no filter in the receiver. For this case, bypass the receive filter
% by doing r = s_rx in the function DPLLMnMClockRecovery()
clear all
Gp = 250;         % Loop filter proportional gain
Gi = 250;         % Loop filter integral gain
sps = 12;         % Samples per FM0 symbol
nSymbs = 200;      % Symbols per frame
EbN0 = -5:0.5:25;
dc_offset = 0;
phase_shift = 0;
delay = 0;
beta = 0.5;
span = 8;
shape = 'Square root';
tx_filter = 0;
rx_filter = 3;
k = 0.5; % Bits per symbol (1 bit (FM0 symbol) = 2 BPSK symbols)
snrdB = EbN0+10*log10(k)-10*log10(sps/2);
M = 20000;
% Find true start of frame just once oustside the loop as BER is computed only on payload data
[data,s_rx,s_fm0] = GenRandFM0Frame(sps,nSymbs,100,tx_filter,dc_offset,phase_shift,beta,span,shape);
s_inphase=real(s_rx);
% Decode received signal using hard decision 
[s_clk,s_match] = DPLLMnMClockRecovery(s_inphase,sps,Gp,Gi,rx_filter,span,shape,beta);
pay_start = SymbolRateFM0FrameSync(s_clk);  % 
for n = 1:length(snrdB)
    % Reset the error and bit counters
    numErrs = 0;
    numBits = 0;
%     while(numErrs < 200 && numBits < 1e7)
    for m=1:M
        % Generate arbitrary FM0 data
        [data,s_rx,s_fm0] = GenRandFM0Frame(sps,nSymbs,snrdB(n),tx_filter,dc_offset,phase_shift,beta,span,shape);
        s_inphase=real(s_rx);
        % Decode received signal
        [s_clk,s_match] = DPLLMnMClockRecovery(s_inphase,sps,Gp,Gi,rx_filter,span,shape,beta);
        dec_data = FM0SymbolDemap(s_clk,pay_start,nSymbs);
        % Calculate the number of bit errors
        nErrors = biterr(data,dec_data);
        % Increment the error and bit counters
        numErrs = numErrs + nErrors;
        numBits = numBits + nSymbs; % 1 FM0 symbol = 1 bit
    end
    % Estimate the BER
    ber_mean(n) = numErrs/numBits;
end
figure(2)
ber = [EbN0; ber_mean];
% save('ber_DPLL_NoRxfilterFinal.mat','ber');
ber = load('ber_DPLL_NoRxfilterFinal.mat').ber;
semilogy(ber(1,:), ber(2,:),'ok')
B=4
 
% Zero crossing demodulation
clear all
Gp = 250;         % Loop filter proportional gain
Gi = 250;         % Loop filter integral gain
sps = 12;         % Samples per FM0 symbol
nSymbs = 200;      % Symbols per frame
EbN0 = -5:0.5:25;
dc_offset = 0;
phase_shift = 0;
delay = 0;
beta = 0.5;
span = 8;
shape = 'Square root';
txrx_filter = 0;
k = 0.5; % Bits per symbol (1 bit (FM0 symbol) = 2 BPSK symbols)
snrdB = EbN0+10*log10(k)-10*log10(sps/2);
M = 20000;
% Find true start of frame just once oustside the loop as BER is computed only on payload data
% % start=33; % for sps = 12 and txrx_filter = 0
for n = 1:length(snrdB)
    % Reset the error and bit counters
    numErrs = 0;
    numBits = 0;
%     while(numErrs < 200 && numBits < 1e7)
    for m = 1:M
        % Generate arbitrary FM0 data
        [data,s_rx,s_fm0] = GenRandFM0Frame(sps,nSymbs,snrdB(n),txrx_filter,dc_offset,phase_shift,beta,span,shape);
        s_inphase = real(s_rx);
        % Decode received signal
        dec_data = ZCrossFM0Demod(s_inphase,sps,nSymbs);
        % Calculate the number of bit errors
        nErrors = biterr(data,dec_data);
        % Increment the error and bit counters
        numErrs = numErrs + nErrors;
        numBits = numBits + nSymbs; % 1 FM0 symbol = 1 bit
    end
    % Estimate the BER
    ber_mean(n) = numErrs/numBits;
end
figure(2)
ber = [EbN0; ber_mean];
save('ber_ZC1_rectfilterFinal.mat','ber');
ber = load('ber_ZC1_rectfilterFinal.mat').ber;
semilogy(ber(1,:), ber(2,:),'x-r')
B=5
 