function [pay_start,s_corr] = SampleRateFM0FrameSync_2(s_rx,sps)
% This function locates the FM0 preamble pattern in the received signal in the 
% sampling rate domain and returns the start position of the payload data
%   Inputs:
%       s_rx - received signal
%       sps - number of samples per symbol
%   Outputs:
%       pay_start - start of payload data
%       s_corr - cross-correlation score
%   Author: Alirio Boaventura

    s1 = ones(1,sps);                           % FM0 symbol '1'
    if(mod(sps,2)== 0)                          % Even sps
        s0 = [ones(1, sps/2), -ones(1, sps/2)]; % FM0 symbol '1'
    else                                        % Odd sps
        s0 = [ones(1, floor(sps/2)), ...
            0, -ones(1, floor(sps/2))];          % FM0 symbol '0'
    end
%   fm0_preamb = [s1 -s0 -s1 s0 -s1 s1];        % Original FM0 preamble pattern
    fm0_preamb = [-s0 -s1 s0 -s1];           % Shorter FM0 preamble pattern

    % Perform cross-correlation efficiently by using filter whose impulse 
    % response corresponds to the time-reversed preamble pattern
    s_corr = conv(s_rx,flip(fm0_preamb));
    s_corr = s_corr./max(s_corr);   % Normalize
    pay_start = (find(s_corr == max(s_corr))) + 1; % Start position of payload data 
end