function [payload_start] = SymbolRateFM0FrameSync(s_clk,stop,k)
%   This function searches for the FM0 preamble pattern in the clock signal using an efficient
%   recursive approach and returns the start position of the payload data in the received signal
%   Inputs:
%       s_clk - previously recovered clock signal
%       stop - recursive control variable
%       k - sample index
%   Outputs:
%       payload_start - start of payload data
%   Author: Alirio Boaventura

    MaxIters = 100; % Maximum iterations searching for preamble
    if nargin == 1, k = 1; stop = 0; end
    if ~stop % Search for preamble symbols
        P(1) = (s_clk(k)>0 && s_clk(k+1)>0) || (s_clk(k)<0 && s_clk(k+1)<0);          % Symbol '1'
        P(2) = (s_clk(k+2)>0 && s_clk(k+3)<0) || (s_clk(k+2)<0 && s_clk(k+3)>0);      % Symbol '0'
        P(3) = (s_clk(k+4)>0 && s_clk(k+5)>0) || (s_clk(k+4)<0 && s_clk(k+5)<0);      % Symbol '1'
        P(4) = (s_clk(k+6)>0 && s_clk(k+7)<0) || (s_clk(k+6)<0 && s_clk(k+7)>0);      % Symbol '0'
        P(5) = (s_clk(k+8)>0 && s_clk(k+9)>0) || (s_clk(k+8)<0 && s_clk(k+9)<0);      % Symbol 'V'
        P(6) = (s_clk(k+10)>0 && s_clk(k+11)>0) || (s_clk(k+10)<0 && s_clk(k+11)<0);  % Symbol '1'
        stop = ((sum(P) == length(P)) || (k > MaxIters) || (k+11 > length(s_clk)));
        payload_start = SymbolRateFM0FrameSync(s_clk,stop,k+1);
    else    % Return start of payload data
        payload_start = k + 11;
    end
end