% This script evaluates aspects of the IS0 18000-63 baseband transmit logic.
% The baseband data containing the reader command are encoded using pulse interval encoding (PIE),
% modulated using either ASK, SSB-ASK or PR-ASK and upsampled/interpolated using a raised-cosine filter.
% Author: Alirio Boaventura


% Pulse interval encoding and baseband pulse shaping
clear all
rng default % reset random generator
data = randi([0 1],1,100);
mod_type = 0;
command_type = 1;
tari = 6.25e-6;                 
Fs = 2.56e6;
beta = 0;
norm = 1;
tx_filter = 0;
span = 8;
shape = 'Normal';
rng default % reset random generator
s_pie = real(EncodeModulateInterpolate(data,mod_type,command_type,beta,tari,tx_filter,span,shape,norm));
figure
subplot(2,1,1)
plot(1:tari*Fs/2:length(s_pie),s_pie(1:tari*Fs/2:end),'o','color',[0,0,0]+0,'LineWidth',1,'MarkerSize',4)
axis([0 16*25-1 -0.5 1.5])
hold on
plot(s_pie,':','color',[0,0,0]+0.6,'LineWidth',1.5)

tx_filter = 1;
beta = 0.2;
rng default % reset random generator

s_pie = real(EncodeModulateInterpolate(data,mod_type,command_type,beta,tari,tx_filter,span,shape,norm));
plot(real(s_pie),'-','color',[0,0,0]+0.2,'LineWidth',0.8,'MarkerSize',5)
beta = 0.5;
s_pie = real(EncodeModulateInterpolate(data,mod_type,command_type,beta,tari,tx_filter,span,shape,norm));
plot(real(s_pie),'-','color',[0,0,0]+0.5,'LineWidth',0.8,'MarkerSize',5)
beta = 0.9;
s_pie = real(EncodeModulateInterpolate(data,mod_type,command_type,beta,tari,tx_filter,span,shape,norm));
plot(real(s_pie),'-','color',[0,0,0]+0.8,'LineWidth',0.8,'MarkerSize',5)
xlabel('Time (k)')
ylabel('Magnitude (V)')
text(165,-0.25,'(a)')

tx_filter = 0;
s_pie = real(EncodeModulateInterpolate(data,mod_type,command_type,beta,tari,tx_filter,span,shape,norm));
subplot(2,1,2)
stem(1:tari*Fs/2:length(s_pie),s_pie(1:tari*Fs/2:end),'o','color',[0,0,0]+0,'LineWidth',1,'MarkerSize',4)
axis([209 305 -0.5 1.5])
hold on
plot(s_pie,':','color',[0,0,0]+0.6,'LineWidth',1.5)
tx_filter = 1;
beta = 0.2;
s_pie = real(EncodeModulateInterpolate(data,mod_type,command_type,beta,tari,tx_filter,span,shape,norm));
plot(real(s_pie),'-','color',[0,0,0]+0.2,'LineWidth',0.8,'MarkerSize',5)
beta = 0.5;
s_pie = real(EncodeModulateInterpolate(data,mod_type,command_type,beta,tari,tx_filter,span,shape,norm));
plot(real(s_pie),'-','color',[0,0,0]+0.5,'LineWidth',0.8,'MarkerSize',5)
beta = 0.9;
s_pie = real(EncodeModulateInterpolate(data,mod_type,command_type,beta,tari,tx_filter,span,shape,norm));
plot(real(s_pie),'-','color',[0,0,0]+0.8,'LineWidth',0.8,'MarkerSize',5)
xlabel('Time (k)')
ylabel('Magnitude (V)')
text(248,-0.25,'(b)')
legend('PIE coded data','Rect pulse filter','RC filter beta = 0.2','RC filter beta = 0.5','RC filter beta = 0.9')


% DSB-ASK vrs. SSB-ASK
clear all
rng default % reset random generator
R=50;
data = randi([0 1],1,10000);
mod_type = 2;
command_type = 1;
beta = 0.9;
tari = 6.25e-6;                 
Fs = 2.56e6;
tx_filter = 1;
span = 8;
shape = 'Normal';
norm = 0;
s_pie = EncodeModulateInterpolate(data,mod_type,command_type,beta,tari,tx_filter,span,shape,norm);
%s_pie=s_pie(200:end);
s_h = hilbert(real(s_pie));
figure
subplot(2,1,1)
plot(real(s_pie),'color',[0,0,0]+0,'LineWidth',1.5)
hold on
plot(imag(s_h),'color',[0,0,0]+0.6,'LineWidth',1.5)
hold on
plot(imag(s_pie),':','color',[0,0,0]+0.6,'LineWidth',2.5)
legend('Real','Ideal imaginary','Approximate imaginary')
xlabel('Time (k)')
ylabel('Magnitude (V)')
axis([0 tari*Fs*30 -0.5 0.5])
text(10,0.3,'(a)')

N=length(s_pie);
f=Fs/2*[-1:2/N:1-2/N]';
df=Fs/N; % Frequency resolution
% Normalized FFT of DSB signal
Sdsb=(fftshift(fft((real(s_pie)),N))/N);
% Power in Watt on a 50 Ohm load
PdsbW=abs(Sdsb).^2/R;
% power spectrum in dBm
Sdsb=10*log10(PdsbW*1000);
% Power spectral density (PSD) in dBm/Hz
Pxxdsb=10*log10(PdsbW*1000/df);

% Normalized FFT of SSB signal
Sssb=(fftshift(fft((s_pie),N))/N);
% Power in Watt in a 50 Ohm load
PssbW=abs(Sssb).^2/R;
% power spectrum in dBm
Sssb=10*log10(PssbW*1000);
% Power spectral density (PSD) in dBm/Hz
Pxxssb=10*log10(PssbW*1000/df);

subplot(2,1,2)
plot(f*1e-6,Pxxdsb,'color',[0,0,0]+0,'LineWidth',1.5)
hold on
plot(f*1e-6,Pxxssb,'color',[0,0,0]+0.6,'LineWidth',1.5)
xlabel('Frequency (MHz)')
ylabel('PSD (dBm/Hz)')
legend('DSB-ASK','SSB-ASK')
grid on
text(-0.37,-40,'(b)')
axis([-0.4 0.4 -120 0])

% DSB-ASK vrs. PR-ASK
mod_type = 0;
s_dsb = EncodeModulateInterpolate(data,mod_type,command_type,beta,tari,tx_filter,span,shape,norm);

mod_type = 1;
s_pr = EncodeModulateInterpolate(data,mod_type,command_type,beta,tari,tx_filter,span,shape,norm);

figure
subplot(2,1,1)
plot(real(s_dsb),':','color',[0,0,0]+0,'LineWidth',2.5)
hold on
plot(real(s_pr),'color',[0,0,0]+0.6,'LineWidth',1.5)
legend('DSB-ASK','PR-ASK')
xlabel('Time (k)')
ylabel('Magnitude (V)')
axis([0 tari*Fs*30 -0.5 0.5])
text(10,0.4,'(a)')

N=length(s_dsb);
f=Fs/2*[-1:2/N:1-2/N]';
df=Fs/N; % Frequency resolution
% Normalized FFT of DSB signal
Sdsb=(fftshift(fft(s_dsb,N))/N);
% Power in Watt in a 50 Ohm load
PdsbW=abs(Sdsb).^2/R;
% power spectrum in dBm
Sdsb=10*log10(PdsbW*1000);
% Power spectral density (PSD) in dBm/Hz
Pxxdsb=10*log10(PdsbW*1000/df);

N=length(s_pr);
% Normalized FFT of SSB signal
Spr=(fftshift(fft((s_pr),N))/N);
% Power in Watt in a 50 Ohm load
PprW=abs(Spr).^2/R;
% power spectrum in dBm
Spr=10*log10(PprW*1000);
% Power spectral density (PSD) in dBm/Hz
Pxxpr=10*log10(PprW*1000/df);

%figure
subplot(2,1,2)
plot(f*1e-6,Pxxdsb,'color',[0,0,0]+0,'LineWidth',1.5)
hold on
plot(f*1e-6,Pxxpr,'color',[0,0,0]+0.6,'LineWidth',1.5)
xlabel('Frequency (MHz)')
ylabel('PSD (dBm/Hz)')
legend('DSB-ASK','PR-ASK')
grid on
text(-0.37,-40,'(b)')
axis([-0.4 0.4 -120 0])


%% Spectrum mask
% Unfiltered PIE-encoded, DSB-ASK-modulated
clear all
rng default % reset random generator
R=50;
data = randi([0 1],1,10000);
command_type = 1;
beta = 0.5;
tari = 6.25e-6;                 
Fs = 2*2.56e6;
mod_type = 0;
tx_filter = 0;
span = 8;
shape = 'Normal';
norm = 0;
s_dsb = EncodeModulateInterpolate2(data,mod_type,command_type,beta,tari,tx_filter,span,shape,norm,Fs);
N=length(s_dsb)
Ts=1/Fs;
t=0:Ts:(N-1)*Ts;
T=t(end);
ChBw=500e3;
f0 = 0;
f1=f0+ChBw/2;
f2=f1+ChBw;
f3=f2+ChBw;
f4=f3+ChBw;
f5=f4+ChBw;
f=Fs/2*[-1:2/N:1-2/N]';
df=Fs/N; % Frequency resolution
% Normalized FFT of DSB signal
Sdsb=(fftshift(fft(s_dsb,N))/N);
% Power in Watt on a 50 Ohm load
PdsbW=abs(Sdsb).^2/R;
% power spectrum in dBm
SdsbdBm=10*log10(PdsbW*1000);
% Power spectral density (PSD) in dBm/Hz
Pxxdsb=10*log10(PdsbW*1000/df);

% %Average power in the time and frequency domain
% Pav_method1 = bandpower(PdsbW/df,f,[f(1) f(end)],'psd')
% Pav_method2 = bandpower(s_dsb)/R
% Pav_method3 = sum(s_dsb.^2)/(N*R)

figure
plot(f*1e-6,Pxxdsb,'color',[0,0,0]+0.5,'LineWidth',1.5)
xlabel('Frequency (MHz)')
ylabel('PSD (dBm/Hz)')
hold on
grid on

P0 = bandpower(PdsbW/df,f,[-f1 f1],'psd');
P0dbm = 10*log10(P0/df)+30
P1 = bandpower(PdsbW/df,f,[f1 f2],'psd');
P1dbm = 10*log10(P1/df)+30;
ACPR1 = P1dbm-P0dbm
Pm1 = bandpower(PdsbW/df,f,[-f2 -f1],'psd');
Pm1dbm = 10*log10(Pm1/df)+30;
ACPRm1 = Pm1dbm-P0dbm

P2 = bandpower(PdsbW/df,f,[f2 f3],'psd');
P2dbm = 10*log10(P2/df)+30;
ACPR2 = P2dbm-P0dbm
Pm2 = bandpower(PdsbW/df,f,[-f3 -f2],'psd');
Pm2dbm = 10*log10(Pm2/df)+30;
ACPRm2 = Pm2dbm-P0dbm

P3 = bandpower(PdsbW/df,f,[f3 f4],'psd');
P3dbm = 10*log10(P3/df)+30;
ACPR3 = P3dbm-P0dbm
Pm3 = bandpower(PdsbW/df,f,[-f4 -f3],'psd');
Pm3dbm = 10*log10(Pm3/df)+30;
ACPRm3 = Pm3dbm-P0dbm

P4 = bandpower(PdsbW/df,f,[f4 f5],'psd');
P4dbm = 10*log10(P4/df)+30;
ACPR4 = P4dbm-P0dbm
Pm4 = bandpower(PdsbW/df,f,[-f5 -f4],'psd');
Pm4dbm = 10*log10(Pm4/df)+30;
ACPRm4 = Pm4dbm-P0dbm

plot([-f5 -f4 -f4 -f3 -f3 -f2 -f2 -f1 -f1 f1 f1 f2 f2 f3 f3 f4 f4 f5]*1e-6,...
    [Pm4dbm Pm4dbm Pm3dbm Pm3dbm Pm2dbm Pm2dbm Pm1dbm Pm1dbm P0dbm P0dbm...
    P1dbm P1dbm P2dbm P2dbm P3dbm P3dbm Pm4dbm Pm4dbm]...
    ,'-','color',[0,0,0]+0,'LineWidth',1.5)

x0 = 10;
y0 = 10;
width = 620;
height = 370;
set(gcf,'position',[x0,y0,width,height]);

% Plot ISO multiple interrogator environment mask 
plot([-f5 -f4 -f4 -f3 -f3 -f2 -f2 -f1 -f1 f1 f1 f2 f2 f3 f3 f4 f4 f5]*1e-6,...
    [P0dbm-65 P0dbm-65 P0dbm-60 P0dbm-60 P0dbm-50 P0dbm-50 P0dbm-20 P0dbm-20 P0dbm P0dbm ...
    P0dbm-20 P0dbm-20 P0dbm-50 P0dbm-50 P0dbm-60 P0dbm-60 P0dbm-65 P0dbm-65]...
    ,':','color',[0,0,0]+0.7,'LineWidth',1.5)
axis([-f5*1e-6 f5*1e-6 -120 10]);
legend('Power spectral density','Integrated channel power','Multiple interrogator mask')


% Filtered PIE-encoded, DSB-ASK-modulated
clear all
rng default % reset random generator
R=50;
data = randi([0 1],1,10000);
command_type = 1;
beta = 0.53;
tari = 6.25e-6;                 
Fs = 2*2.56e6;
mod_type = 0;
tx_filter = 1;
span = 8;
shape = 'Normal';
norm = 0;
s_dsb = EncodeModulateInterpolate2(data,mod_type,command_type,beta,tari,tx_filter,span,shape,norm,Fs);
N=length(s_dsb)
Ts=1/Fs;
t=0:Ts:(N-1)*Ts;
T=t(end);
ChBw=500e3;
f0 = 0;
f1=f0+ChBw/2;
f2=f1+ChBw;
f3=f2+ChBw;
f4=f3+ChBw;
f5=f4+ChBw;
f=Fs/2*[-1:2/N:1-2/N]';
df=Fs/N; % Frequency resolution
% Normalized FFT of DSB signal
Sdsb=(fftshift(fft(s_dsb,N))/N);
% Power in Watt on a 50 Ohm load
PdsbW=abs(Sdsb).^2/R;
% power spectrum in dBm
SdsbdBm=10*log10(PdsbW*1000);
% Power spectral density (PSD) in dBm/Hz
Pxxdsb=10*log10(PdsbW*1000/df);

% %Average power in the time and frequency domain
% Pav_method1 = bandpower(PdsbW/df,f,[f(1) f(end)],'psd')
% Pav_method2 = bandpower(s_dsb)/R
% Pav_method3 = sum(s_dsb.^2)/(N*R)

figure
plot(f*1e-6,Pxxdsb,'color',[0,0,0]+0.5,'LineWidth',1.5)
xlabel('Frequency (MHz)')
ylabel('PSD (dBm/Hz)')
hold on
grid on

P0 = bandpower(PdsbW/df,f,[-f1 f1],'psd');
P0dbm = 10*log10(P0/df)+30
P1 = bandpower(PdsbW/df,f,[f1 f2],'psd');
P1dbm = 10*log10(P1/df)+30;
ACPR1 = P1dbm-P0dbm
Pm1 = bandpower(PdsbW/df,f,[-f2 -f1],'psd');
Pm1dbm = 10*log10(Pm1/df)+30;
ACPRm1 = Pm1dbm-P0dbm

P2 = bandpower(PdsbW/df,f,[f2 f3],'psd');
P2dbm = 10*log10(P2/df)+30;
ACPR2 = P2dbm-P0dbm
Pm2 = bandpower(PdsbW/df,f,[-f3 -f2],'psd');
Pm2dbm = 10*log10(Pm2/df)+30;
ACPRm2 = Pm2dbm-P0dbm

P3 = bandpower(PdsbW/df,f,[f3 f4],'psd');
P3dbm = 10*log10(P3/df)+30;
ACPR3 = P3dbm-P0dbm
Pm3 = bandpower(PdsbW/df,f,[-f4 -f3],'psd');
Pm3dbm = 10*log10(Pm3/df)+30;
ACPRm3 = Pm3dbm-P0dbm

P4 = bandpower(PdsbW/df,f,[f4 f5],'psd');
P4dbm = 10*log10(P4/df)+30;
ACPR4 = P4dbm-P0dbm
Pm4 = bandpower(PdsbW/df,f,[-f5 -f4],'psd');
Pm4dbm = 10*log10(Pm4/df)+30;
ACPRm4 = Pm4dbm-P0dbm

plot([-f5 -f4 -f4 -f3 -f3 -f2 -f2 -f1 -f1 f1 f1 f2 f2 f3 f3 f4 f4 f5]*1e-6,...
    [Pm4dbm Pm4dbm Pm3dbm Pm3dbm Pm2dbm Pm2dbm Pm1dbm Pm1dbm P0dbm P0dbm...
    P1dbm P1dbm P2dbm P2dbm P3dbm P3dbm Pm4dbm Pm4dbm]...
    ,'-','color',[0,0,0]+0,'LineWidth',1.5)

x0 = 10;
y0 = 10;
width = 620;
height = 370;
set(gcf,'position',[x0,y0,width,height]);

% Plot ISO multiple interrogator environment mask 
plot([-f5 -f4 -f4 -f3 -f3 -f2 -f2 -f1 -f1 f1 f1 f2 f2 f3 f3 f4 f4 f5]*1e-6,...
    [P0dbm-65 P0dbm-65 P0dbm-60 P0dbm-60 P0dbm-50 P0dbm-50 P0dbm-20 P0dbm-20 P0dbm P0dbm ...
    P0dbm-20 P0dbm-20 P0dbm-50 P0dbm-50 P0dbm-60 P0dbm-60 P0dbm-65 P0dbm-65]...
    ,':','color',[0,0,0]+0.7,'LineWidth',1.5)
axis([-f5*1e-6 f5*1e-6 -140 0]);
legend('Power spectral density','Integrated channel power','Multiple interrogator mask')


% Filtered PIE-encoded, PR-ASK-modulated
clear all
rng default % reset random generator
R=50;
data = randi([0 1],1,10000);
command_type = 1;
beta = 0.2;
tari = 6.25e-6;                 
Fs = 2*2.56e6;
mod_type = 1;
tx_filter = 1;
span = 16;
shape = 'Normal';
norm = 0;
s_pr = EncodeModulateInterpolate2(data,mod_type,command_type,beta,tari,tx_filter,span,shape,norm,Fs);
N=length(s_pr)
Ts=1/Fs;
t=0:Ts:(N-1)*Ts;
T=t(end);
ChBw=250e3;
f0 = 0;
f1=f0+ChBw/2;
f2=f1+ChBw;
f3=f2+ChBw;
f4=f3+ChBw;
f5=f4+ChBw;
f=Fs/2*[-1:2/N:1-2/N]';
%f4=f3+ChBw;
df=Fs/N; % Frequency resolution
% Normalized FFT of DSB signal
Sdsb=(fftshift(fft(s_pr,N))/N);
% Power in Watt on a 50 Ohm load
PdsbW=abs(Sdsb).^2/R;
% power spectrum in dBm
SdsbdBm=10*log10(PdsbW*1000);
% Power spectral density (PSD) in dBm/Hz
Pxxdsb=10*log10(PdsbW*1000/df);

% %Average power in the time and frequency domain
% Pav_method1 = bandpower(PdsbW/df,f,[f(1) f(end)],'psd')
% Pav_method2 = bandpower(s_dsb)/R
% Pav_method3 = sum(s_dsb.^2)/(N*R)

figure
plot(f*1e-6,Pxxdsb,'color',[0,0,0]+0.5,'LineWidth',1.5)
xlabel('Frequency (MHz)')
ylabel('PSD (dBm/Hz)')
hold on
grid on

P0 = bandpower(PdsbW/df,f,[-f1 f1],'psd');
P0dbm = 10*log10(P0/df)+30
P1 = bandpower(PdsbW/df,f,[f1 f2],'psd');
P1dbm = 10*log10(P1/df)+30;
ACPR1 = P1dbm-P0dbm
Pm1 = bandpower(PdsbW/df,f,[-f2 -f1],'psd');
Pm1dbm = 10*log10(Pm1/df)+30;
ACPRm1 = Pm1dbm-P0dbm

P2 = bandpower(PdsbW/df,f,[f2 f3],'psd');
P2dbm = 10*log10(P2/df)+30;
ACPR2 = P2dbm-P0dbm
Pm2 = bandpower(PdsbW/df,f,[-f3 -f2],'psd');
Pm2dbm = 10*log10(Pm2/df)+30;
ACPRm2 = Pm2dbm-P0dbm

P3 = bandpower(PdsbW/df,f,[f3 f4],'psd');
P3dbm = 10*log10(P3/df)+30;
ACPR3 = P3dbm-P0dbm
Pm3 = bandpower(PdsbW/df,f,[-f4 -f3],'psd');
Pm3dbm = 10*log10(Pm3/df)+30;
ACPRm3 = Pm3dbm-P0dbm

P4 = bandpower(PdsbW/df,f,[f4 f5],'psd');
P4dbm = 10*log10(P4/df)+30;
ACPR4 = P4dbm-P0dbm
Pm4 = bandpower(PdsbW/df,f,[-f5 -f4],'psd');
Pm4dbm = 10*log10(Pm4/df)+30;
ACPRm4 = Pm4dbm-P0dbm

plot([-f5 -f4 -f4 -f3 -f3 -f2 -f2 -f1 -f1 f1 f1 f2 f2 f3 f3 f4 f4 f5]*1e-6,...
    [Pm4dbm Pm4dbm Pm3dbm Pm3dbm Pm2dbm Pm2dbm Pm1dbm Pm1dbm P0dbm P0dbm...
    P1dbm P1dbm P2dbm P2dbm P3dbm P3dbm Pm4dbm Pm4dbm]...
    ,'-','color',[0,0,0]+0,'LineWidth',1.5)

x0 = 10;
y0 = 10;
width = 620;
height = 370;
set(gcf,'position',[x0,y0,width,height]);

% Plot ISO multiple interrogator environment mask 
plot([-f5 -f4 -f4 -f3 -f3 -f2 -f2 -f1 -f1 f1 f1 f2 f2 f3 f3 f4 f4 f5]*1e-6,...
    [P0dbm-65 P0dbm-65 P0dbm-60 P0dbm-60 P0dbm-50 P0dbm-50 P0dbm-20 P0dbm-20 P0dbm P0dbm ...
    P0dbm-20 P0dbm-20 P0dbm-50 P0dbm-50 P0dbm-60 P0dbm-60 P0dbm-65 P0dbm-65]...
    ,':','color',[0,0,0]+0.7,'LineWidth',1.5)
axis([-2.25 2.25 -140 0]);
legend('Power spectral density','Integrated channel power','Multiple interrogator mask')

